package cn.newfeifan.mall.module.product.service.spuapplylog;

import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import cn.newfeifan.mall.module.product.controller.admin.spuapplylog.vo.*;
import cn.newfeifan.mall.module.product.dal.dataobject.spuapplylog.SpuApplyLogDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.util.object.BeanUtils;

import cn.newfeifan.mall.module.product.dal.mysql.spuapplylog.SpuApplyLogMapper;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.module.product.enums.ErrorCodeConstants.*;

/**
 * 商品审核记录 Service 实现类
 *
 * @author 非繁人
 */
@Service
@Validated
public class SpuApplyLogServiceImpl implements SpuApplyLogService {

    @Resource
    private SpuApplyLogMapper spuApplyLogMapper;

    @Override
    public Long createSpuApplyLog(SpuApplyLogSaveReqVO createReqVO) {
        // 插入
        SpuApplyLogDO spuApplyLog = BeanUtils.toBean(createReqVO, SpuApplyLogDO.class);
        spuApplyLogMapper.insert(spuApplyLog);
        // 返回
        return spuApplyLog.getId();
    }

    @Override
    public void updateSpuApplyLog(SpuApplyLogSaveReqVO updateReqVO) {
        // 校验存在
        validateSpuApplyLogExists(updateReqVO.getId());
        // 更新
        SpuApplyLogDO updateObj = BeanUtils.toBean(updateReqVO, SpuApplyLogDO.class);
        spuApplyLogMapper.updateById(updateObj);
    }

    @Override
    public void deleteSpuApplyLog(Long id) {
        // 校验存在
        validateSpuApplyLogExists(id);
        // 删除
        spuApplyLogMapper.deleteById(id);
    }

    private void validateSpuApplyLogExists(Long id) {
        if (spuApplyLogMapper.selectById(id) == null) {
            throw exception(SPU_APPLY_LOG_NOT_EXISTS);
        }
    }

    @Override
    public SpuApplyLogDO getSpuApplyLog(Long id) {
        return spuApplyLogMapper.selectById(id);
    }

    @Override
    public PageResult<SpuApplyLogDO> getSpuApplyLogPage(SpuApplyLogPageReqVO pageReqVO) {
        return spuApplyLogMapper.selectPage(pageReqVO);
    }

}